@@ -5,31 +5,31 @@ import sqlite3
 # Re-export some names from sqliteparser.
 from sqliteparser.ast import OnConflict, OnDelete, OnUpdate
 
-from .core import AddColumnMigration, AlterColumnMigration, AutoTable
-from .core import BooleanColumnStub as BooleanColumn
 from .core import (
+    AddColumnMigration,
+    AlterColumnMigration,
+    AutoTable,
+    BooleanColumn,
     ColumnDoesNotExistError,
     CreateTableMigration,
     Database,
-    DatabaseMigrator,
-)
-from .core import DateColumnStub as DateColumn
-from .core import DecimalColumnStub as DecimalColumn
-from .core import DropColumnMigration, DropTableMigration
-from .core import ForeignKeyColumnStub as ForeignKeyColumn
-from .core import IntegerColumnStub as IntegerColumn
-from .core import (
+    DateColumn,
+    DecimalColumn,
+    DropColumnMigration,
+    DropTableMigration,
+    ForeignKeyColumn,
+    IntegerColumn,
     ISqliteApiError,
     ISqliteError,
+    PrimaryKeyColumn,
     PrintDebugger,
     ReorderColumnsMigration,
     Table,
     TableDoesNotExistError,
+    TextColumn,
+    TimeColumn,
+    TimestampColumn,
 )
-from .core import TextColumnStub as TextColumn
-from .core import TimeColumnStub as TimeColumn
-from .core import TimestampColumnStub as TimestampColumn
-from .core import schema_module_to_dict
 
 
 def sqlite3_convert_boolean(b):
@@ -67,3 +67,9 @@ sqlite3.register_converter("DECIMAL", sqlite3_convert_decimal)
 sqlite3.register_adapter(decimal.Decimal, sqlite3_adapt_decimal)
 sqlite3.register_converter("TIME", sqlite3_convert_time)
 sqlite3.register_adapter(datetime.time, sqlite3_adapt_time)
+
+del sqlite3_convert_boolean
+del sqlite3_convert_decimal
+del sqlite3_adapt_decimal
+del sqlite3_convert_time
+del sqlite3_adapt_time
