@@ -8,7 +8,6 @@ used by Click, the argument parsing framework. The tests in `tests/test_cli.py`
 """
 import collections
 import importlib
-import readline  # noqa: F401
 import shutil
 import sqlite3
 import sys
@@ -21,20 +20,25 @@ from tabulate import tabulate
 from .core import CreateTableMigration, Database, DropTableMigration
 
 # Help strings used in multiple places.
-COLUMNS_HELP = "Only display these columns in the results."
-HIDE_HELP = "Hide these columns in the results."
-PAGE_HELP = (
-    "Select the page of results to show, "
-    + "if the table is too wide to display in one screen."
-)
-LIMIT_HELP = "Limit the number of rows returned from the database."
-OFFSET_HELP = "Offset a query with --limit."
-ORDER_BY_HELP = "Order the results by one or more columns."
-DESC_HELP = (
+HELP_COLUMNS = "Only display these columns in the results."
+HELP_DESC = (
     "When combined with --order-by, order the results in descending rather than "
     + "ascending order."
 )
-NO_CONFIRM_HELP = "Do not prompt for confirmation."
+HELP_HIDE = "Hide these columns in the results."
+HELP_LIMIT = "Limit the number of rows returned from the database."
+HELP_NO_CONFIRM = "Do not prompt for confirmation."
+HELP_OFFSET = "Offset a query with --limit."
+HELP_ORDER_BY = "Order the results by one or more columns."
+HELP_PAGE = (
+    "Select the page of results to show, "
+    + "if the table is too wide to display in one screen."
+)
+HELP_PLAIN_FOREIGN_KEYS = (
+    "By default, isqlite will pretty-print foreign key columns with the first "
+    + "TEXT column of the foreign key table. Pass this flag if you would rather just "
+    + "see the foreign key values themselves."
+)
 
 
 @click.group()
@@ -43,7 +47,7 @@ def cli():
 
 
 @cli.command(name="add-column")
-@click.option("--db", "db_path", envvar="ISQLITE_DB")
+@click.argument("db_path")
 @click.argument("table")
 @click.argument("column")
 def main_add_column_wrapper(*args, **kwargs):
@@ -60,8 +64,7 @@ def main_add_column(db_path, table, column):
 
 
 @cli.command(name="alter-column")
-@click.option("--db", "db_path", envvar="ISQLITE_DB")
-@click.option("--schema", "schema_path", envvar="ISQLITE_SCHEMA")
+@click.argument("db_path")
 @click.argument("table")
 @click.argument("column")
 def main_alter_column_wrapper(*args, **kwargs):
@@ -71,16 +74,15 @@ def main_alter_column_wrapper(*args, **kwargs):
     return main_alter_column(*args, **kwargs)
 
 
-def main_alter_column(db_path, schema_path, table, column):
-    schema = get_schema_from_path(schema_path)
-    with Database(db_path, schema=schema) as db:
+def main_alter_column(db_path, table, column):
+    with Database(db_path) as db:
         column_name, column_def = column.split(maxsplit=1)
         db.alter_column(table, column_name, column_def)
         print(f"Column {column_name!r} altered in table {table!r}.")
 
 
 @cli.command(name="create")
-@click.option("--db", "db_path", envvar="ISQLITE_DB")
+@click.argument("db_path")
 @click.argument("table")
 @click.argument("payload", nargs=-1)
 @click.option(
@@ -113,44 +115,20 @@ def main_create(db_path, table, payload, *, auto_timestamp=True):
         key, value = key_value.split("=")
         payload_as_map[key] = value
 
+    if auto_timestamp:
+        auto_timestamp_columns = ["created_at", "last_updated_at"]
+    else:
+        auto_timestamp_columns = []
+
     with Database(db_path) as db:
-        pk = db.create(table, payload_as_map, auto_timestamp=auto_timestamp)
-        print(f"Row {pk} created.")
-
-
-@cli.command(name="icreate")
-@click.option("--db", "db_path", envvar="ISQLITE_DB")
-@click.option("--schema", "schema_path", envvar="ISQLITE_SCHEMA")
-@click.argument("table")
-def main_icreate(db_path, schema_path, table):
-    """
-    Create a new row interactively.
-    """
-    schema = get_schema_map_from_path(schema_path)
-    with Database(db_path) as db:
-        table_schema = schema.get(table)
-        if table_schema is None:
-            print(f"Table {table!r} not found in schema.")
-            sys.exit(1)
-
-        payload = {}
-        for column in table_schema.columns.values():
-            if column.name in ("id", "created_at", "last_updated_at"):
-                continue
-
-            while True:
-                raw_value = input(f"{column.description()}? ").strip()
-                value, is_valid = column.validate(raw_value)
-                if is_valid:
-                    payload[column.name] = value
-                    break
-
-        pk = db.create(table, payload)
+        pk = db.create(
+            table, payload_as_map, auto_timestamp_columns=auto_timestamp_columns
+        )
         print(f"Row {pk} created.")
 
 
 @cli.command(name="create-table")
-@click.option("--db", "db_path", envvar="ISQLITE_DB")
+@click.argument("db_path")
 @click.argument("table")
 @click.argument("columns", nargs=-1)
 def main_create_table_wrapper(*args, **kwargs):
@@ -167,7 +145,7 @@ def main_create_table(db_path, table, columns):
 
 
 @cli.command(name="delete")
-@click.option("--db", "db_path", envvar="ISQLITE_DB")
+@click.argument("db_path")
 @click.argument("table")
 @click.argument("pk", type=int, required=False, default=None)
 @click.option("-w", "--where", default="")
@@ -175,7 +153,7 @@ def main_create_table(db_path, table, columns):
     "--no-confirm",
     is_flag=True,
     default=False,
-    help=NO_CONFIRM_HELP,
+    help=HELP_NO_CONFIRM,
 )
 def main_delete_wrapper(*args, **kwargs):
     """
@@ -237,15 +215,14 @@ def main_delete(db_path, table, pk=None, *, where="", no_confirm=False):
 
 
 @cli.command(name="drop-column")
-@click.option("--db", "db_path", envvar="ISQLITE_DB")
-@click.option("--schema", "schema_path", envvar="ISQLITE_SCHEMA")
+@click.argument("db_path")
 @click.argument("table")
 @click.argument("column")
 @click.option(
     "--no-confirm",
     is_flag=True,
     default=False,
-    help=NO_CONFIRM_HELP,
+    help=HELP_NO_CONFIRM,
 )
 def main_drop_column_wrapper(*args, **kwargs):
     """
@@ -254,8 +231,7 @@ def main_drop_column_wrapper(*args, **kwargs):
     main_drop_column(*args, **kwargs)
 
 
-def main_drop_column(db_path, schema_path, table, column, *, no_confirm=False):
-    schema = get_schema_from_path(schema_path)
+def main_drop_column(db_path, table, column, *, no_confirm=False):
     if not no_confirm:
         with Database(db_path) as db:
             count = db.count(table)
@@ -267,20 +243,20 @@ def main_drop_column(db_path, schema_path, table, column, *, no_confirm=False):
                 print("Operation aborted.")
                 sys.exit(1)
 
-    with Database(db_path, schema=schema) as db:
+    with Database(db_path) as db:
         db.drop_column(table, column)
         print()
         print(f"Column {column!r} dropped from table {table!r}.")
 
 
 @cli.command(name="drop-table")
-@click.option("--db", "db_path", envvar="ISQLITE_DB")
+@click.argument("db_path")
 @click.argument("table")
 @click.option(
     "--no-confirm",
     is_flag=True,
     default=False,
-    help=NO_CONFIRM_HELP,
+    help=HELP_NO_CONFIRM,
 )
 def main_drop_table_wrapper(*args, **kwargs):
     """
@@ -306,10 +282,12 @@ def main_drop_table(db_path, table, *, no_confirm=False):
 
 
 @cli.command(name="get")
-@click.option("--db", "db_path", envvar="ISQLITE_DB")
-@click.option("--schema", "schema_path", envvar="ISQLITE_SCHEMA")
+@click.argument("db_path")
 @click.argument("table")
 @click.argument("pk", type=int)
+@click.option(
+    "--plain-foreign-keys", is_flag=True, default=False, help=HELP_PLAIN_FOREIGN_KEYS
+)
 def main_get_wrapper(*args, **kwargs):
     """
     Fetch a single row.
@@ -317,18 +295,13 @@ def main_get_wrapper(*args, **kwargs):
     main_get(*args, **kwargs)
 
 
-def main_get(db_path, schema_path, table, pk):
-    if schema_path:
-        schema = get_schema_from_path(schema_path)
-        schema_map = schema_to_map(schema)
-    else:
-        schema = None
-
-    with Database(db_path, readonly=True, schema=schema) as db:
-        row = db.get_by_pk(table, pk, get_related=schema is not None)
-        for key, value in row.items():
-            if isinstance(value, collections.OrderedDict):
-                row[key] = get_column_as_string(schema_map, table, key, value)
+def main_get(db_path, table, pk, *, plain_foreign_keys=False):
+    with Database(db_path, readonly=True) as db:
+        row = db.get_by_pk(table, pk, get_related=not plain_foreign_keys)
+        if not plain_foreign_keys:
+            for key, value in row.items():
+                if isinstance(value, collections.OrderedDict):
+                    row[key] = get_column_as_string(value)
 
         if row is None:
             print(f"Row {pk} not found in table {table!r}.")
@@ -337,18 +310,20 @@ def main_get(db_path, schema_path, table, pk):
 
 
 @cli.command(name="list")
-@click.option("--db", "db_path", envvar="ISQLITE_DB")
-@click.option("--schema", "schema_path", envvar="ISQLITE_SCHEMA")
+@click.argument("db_path")
 @click.argument("table")
 @click.option("-w", "--where", default="")
 @click.option("-s", "--search")
-@click.option("--columns", multiple=True, default=[], help=COLUMNS_HELP)
-@click.option("--hide", multiple=True, default=[], help=HIDE_HELP)
-@click.option("-p", "--page", default=1, help=PAGE_HELP)
-@click.option("--limit", default=None, help=LIMIT_HELP)
-@click.option("--offset", default=None, help=OFFSET_HELP)
-@click.option("--order-by", multiple=True, default=[], help=ORDER_BY_HELP)
-@click.option("--desc", is_flag=True, default=False, help=DESC_HELP)
+@click.option("--columns", multiple=True, default=[], help=HELP_COLUMNS)
+@click.option("--hide", multiple=True, default=[], help=HELP_HIDE)
+@click.option("-p", "--page", default=1, help=HELP_PAGE)
+@click.option("--limit", default=None, help=HELP_LIMIT)
+@click.option("--offset", default=None, help=HELP_OFFSET)
+@click.option("--order-by", multiple=True, default=[], help=HELP_ORDER_BY)
+@click.option("--desc", is_flag=True, default=False, help=HELP_DESC)
+@click.option(
+    "--plain-foreign-keys", is_flag=True, default=False, help=HELP_PLAIN_FOREIGN_KEYS
+)
 def main_list_wrapper(*args, **kwargs):
     """
     List the rows in the table, optionally filtered by a SQL clause.
@@ -358,7 +333,6 @@ def main_list_wrapper(*args, **kwargs):
 
 def main_list(
     db_path,
-    schema_path,
     table,
     *,
     where=None,
@@ -370,10 +344,10 @@ def main_list(
     offset=None,
     order_by=[],
     desc=False,
+    plain_foreign_keys=False,
 ):
     base_list(
         db_path,
-        schema_path,
         table,
         where=where,
         search=search,
@@ -384,12 +358,12 @@ def main_list(
         offset=offset,
         order_by=order_by,
         desc=desc,
+        plain_foreign_keys=plain_foreign_keys,
     )
 
 
 def base_list(
     db_path,
-    schema_path,
     table,
     *,
     where,
@@ -401,17 +375,12 @@ def base_list(
     offset,
     order_by,
     desc,
+    plain_foreign_keys,
 ):
     """
     Base implementation shared by `main_list` and `main_search`
     """
-    if schema_path:
-        schema = get_schema_from_path(schema_path)
-        schema_map = schema_to_map(schema)
-    else:
-        schema = None
-
-    with Database(db_path, readonly=True, schema=schema) as db:
+    with Database(db_path, readonly=True) as db:
         try:
             rows = db.list(
                 table,
@@ -420,8 +389,7 @@ def base_list(
                 limit=limit,
                 offset=offset,
                 descending=desc if order_by else None,
-                # `get_related` is only possible when the database has a schema.
-                get_related=schema is not None,
+                get_related=not plain_foreign_keys,
             )
         except sqlite3.OperationalError:
             # Because `get_related` uses SQL joins, it may cause 'ambiguous column'
@@ -437,10 +405,11 @@ def base_list(
                 get_related=False,
             )
 
-        for row in rows:
-            for key, value in row.items():
-                if isinstance(value, collections.OrderedDict):
-                    row[key] = get_column_as_string(schema_map, table, key, value)
+        if not plain_foreign_keys:
+            for row in rows:
+                for key, value in row.items():
+                    if isinstance(value, collections.OrderedDict):
+                        row[key] = get_column_as_string(value)
 
         if search:
             search = search.lower()
@@ -478,7 +447,7 @@ def base_list(
 
 
 @cli.command(name="list-tables")
-@click.option("--db", "db_path", envvar="ISQLITE_DB")
+@click.argument("db_path")
 def main_list_tables_wrapper(*args, **kwargs):
     """
     List the names of the tables in the database.
@@ -507,8 +476,8 @@ def main_list_tables(db_path):
 
 
 @cli.command(name="migrate")
-@click.option("--db", "db_path", envvar="ISQLITE_DB")
-@click.option("--schema", "schema_path", envvar="ISQLITE_SCHEMA")
+@click.argument("db_path")
+@click.argument("schema_path")
 @click.argument("table", required=False, default=None)
 @click.option(
     "--write",
@@ -539,12 +508,11 @@ def main_migrate(db_path, schema_path, table, *, write, no_backup, debug):
     schema = get_schema_from_path(schema_path)
     with Database(
         db_path,
-        schema=schema,
         readonly=not write,
         transaction=False,
         debugger=debug,
     ) as db:
-        diff = db.diff(table=table)
+        diff = db.diff(schema, table=table)
         if not diff:
             print("Nothing to migrate - database matches schema.")
             return
@@ -628,8 +596,7 @@ def main_migrate(db_path, schema_path, table, *, write, no_backup, debug):
 
 
 @cli.command(name="rename-column")
-@click.option("--db", "db_path", envvar="ISQLITE_DB")
-@click.option("--schema", "schema_path", envvar="ISQLITE_SCHEMA")
+@click.argument("db_path")
 @click.argument("table")
 @click.argument("old_name")
 @click.argument("new_name")
@@ -640,15 +607,14 @@ def main_rename_column_wrapper(*args, **kwargs):
     main_rename_column(*args, **kwargs)
 
 
-def main_rename_column(db_path, schema_path, table, old_name, new_name):
-    schema = get_schema_from_path(schema_path)
-    with Database(db_path, schema=schema) as db:
+def main_rename_column(db_path, table, old_name, new_name):
+    with Database(db_path) as db:
         db.rename_column(table, old_name, new_name)
         print(f"Column {old_name!r} renamed to {new_name!r} in table {table!r}.")
 
 
 @cli.command(name="rename-table")
-@click.option("--db", "db_path", envvar="ISQLITE_DB")
+@click.argument("db_path")
 @click.argument("table")
 @click.argument("new_name")
 def main_rename_table_wrapper(*args, **kwargs):
@@ -665,8 +631,7 @@ def main_rename_table(db_path, table, new_name):
 
 
 @cli.command(name="reorder-columns")
-@click.option("--db", "db_path", envvar="ISQLITE_DB")
-@click.option("--schema", "schema_path", envvar="ISQLITE_SCHEMA")
+@click.argument("db_path")
 @click.argument("table")
 @click.argument("columns", nargs=-1)
 def main_reorder_columns_wrapper(*args, **kwargs):
@@ -676,26 +641,27 @@ def main_reorder_columns_wrapper(*args, **kwargs):
     main_reorder_columns(*args, **kwargs)
 
 
-def main_reorder_columns(db_path, schema_path, table, columns):
-    schema = get_schema_from_path(schema_path)
-    with Database(db_path, schema=schema) as db:
+def main_reorder_columns(db_path, table, columns):
+    with Database(db_path) as db:
         db.reorder_columns(table, columns)
         print(f"Columns of table {table!r} reordered.")
 
 
 @cli.command(name="search")
-@click.option("--db", "db_path", envvar="ISQLITE_DB")
-@click.option("--schema", "schema_path", envvar="ISQLITE_SCHEMA")
+@click.argument("db_path")
 @click.argument("table")
 @click.argument("query")
 @click.option("-w", "--where", default="")
-@click.option("--columns", multiple=True, default=[], help=COLUMNS_HELP)
-@click.option("--hide", multiple=True, default=[], help=HIDE_HELP)
-@click.option("-p", "--page", default=1, help=PAGE_HELP)
-@click.option("--limit", default=None, help=LIMIT_HELP)
-@click.option("--offset", default=None, help=OFFSET_HELP)
-@click.option("--order-by", multiple=True, default=[], help=ORDER_BY_HELP)
-@click.option("--desc", is_flag=True, default=False, help=DESC_HELP)
+@click.option("--columns", multiple=True, default=[], help=HELP_COLUMNS)
+@click.option("--hide", multiple=True, default=[], help=HELP_HIDE)
+@click.option("-p", "--page", default=1, help=HELP_PAGE)
+@click.option("--limit", default=None, help=HELP_LIMIT)
+@click.option("--offset", default=None, help=HELP_OFFSET)
+@click.option("--order-by", multiple=True, default=[], help=HELP_ORDER_BY)
+@click.option("--desc", is_flag=True, default=False, help=HELP_DESC)
+@click.option(
+    "--plain-foreign-keys", is_flag=True, default=False, help=HELP_PLAIN_FOREIGN_KEYS
+)
 def main_search_wrapper(*args, **kwargs):
     """
     Shorthand for `list <table> -s <query>`
@@ -705,7 +671,6 @@ def main_search_wrapper(*args, **kwargs):
 
 def main_search(
     db_path,
-    schema_path,
     table,
     query,
     *,
@@ -717,10 +682,10 @@ def main_search(
     offset=None,
     order_by=[],
     desc=False,
+    plain_foreign_keys=False,
 ):
     base_list(
         db_path,
-        schema_path,
         table,
         where=where,
         search=query,
@@ -731,15 +696,16 @@ def main_search(
         offset=offset,
         order_by=order_by,
         desc=desc,
+        plain_foreign_keys=plain_foreign_keys,
     )
 
 
 @cli.command(name="sql")
-@click.option("--db", "db_path", envvar="ISQLITE_DB")
+@click.argument("db_path")
 @click.argument("query")
-@click.option("--columns", multiple=True, default=[], help=COLUMNS_HELP)
-@click.option("--hide", multiple=True, default=[], help=HIDE_HELP)
-@click.option("-p", "--page", default=1, help=PAGE_HELP)
+@click.option("--columns", multiple=True, default=[], help=HELP_COLUMNS)
+@click.option("--hide", multiple=True, default=[], help=HELP_HIDE)
+@click.option("-p", "--page", default=1, help=HELP_PAGE)
 @click.option(
     "--write",
     is_flag=True,
@@ -760,7 +726,7 @@ def main_sql(db_path, query, *, columns, hide, page, write):
 
 
 @cli.command(name="update")
-@click.option("--db", "db_path", envvar="ISQLITE_DB")
+@click.argument("db_path")
 @click.argument("table")
 @click.argument("pk", type=int)
 @click.argument("payload", nargs=-1)
@@ -791,62 +757,15 @@ def main_update(db_path, table, pk, payload, *, auto_timestamp=True):
         key, value = key_value.split("=")
         payload_as_map[key] = value
 
+    if auto_timestamp:
+        auto_timestamp_columns = ["created_at", "last_updated_at"]
+    else:
+        auto_timestamp_columns = []
+
     with Database(db_path) as db:
-        db.update_by_pk(table, pk, payload_as_map, auto_timestamp=auto_timestamp)
-        print(f"Row {pk} updated.")
-
-
-@cli.command(name="iupdate")
-@click.option("--db", "db_path", envvar="ISQLITE_DB")
-@click.option("--schema", "schema_path", envvar="ISQLITE_SCHEMA")
-@click.argument("table")
-@click.argument("pk", type=int)
-def main_iupdate(db_path, schema_path, table, pk):
-    """
-    Update an existing row interactively.
-    """
-    schema = get_schema_map_from_path(schema_path)
-    with Database(db_path) as db:
-        table_schema = schema.get(table)
-        if table_schema is None:
-            print(f"Table {table!r} not found.")
-            sys.exit(1)
-
-        row = db.get_by_pk(table, pk)
-        if row is None:
-            print(f"Row {pk} not found in table {table!r}.")
-            sys.exit(1)
-
-        print("To keep a column's current value, enter a blank line.")
-        print("To set a column to null, enter NULL in all caps.")
-        print()
-
-        updates = {}
-        for column in table_schema.columns.values():
-            if column.name in ("id", "created_at", "last_updated_at"):
-                continue
-
-            while True:
-                currently = (
-                    "null"
-                    if row[column.name] is None or row[column.name] == ""
-                    else repr(row[column.name])
-                )
-                raw_value = input(
-                    f"{column.description()} (currently: {currently})? "
-                ).strip()
-                if raw_value == "":
-                    break
-                elif raw_value == "NULL":
-                    raw_value = ""
-
-                value, is_valid = column.validate(raw_value)
-                if is_valid:
-                    updates[column.name] = value
-                    break
-
-        db.update_by_pk(table, pk, updates)
-        print()
+        db.update_by_pk(
+            table, pk, payload_as_map, auto_timestamp_columns=auto_timestamp_columns
+        )
         print(f"Row {pk} updated.")
 
 
@@ -904,10 +823,31 @@ def should_show_column(key, columns, hide):
     return True
 
 
-def get_column_as_string(schema_map, base_table, column_name, column_value):
-    base_table_def = schema_map[base_table]
-    table = schema_map[base_table_def.columns[column_name].model]
-    return table.as_string(column_value)
+def get_column_as_string(column):
+    pk = None
+    text = None
+
+    # Pretty rudimentary logic: assume the first integer value is the row's primary key
+    # and the first string value is a reasonable choice for displaying the row.
+    for value in column.values():
+        if isinstance(value, int) and pk is None:
+            pk = value
+        elif isinstance(value, str) and text is None:
+            text = value
+
+        if pk is not None and text is not None:
+            break
+
+    if pk is not None:
+        if text is not None:
+            return f"{pk} ({text})"
+        else:
+            return str(pk)
+    else:
+        if text is not None:
+            return text
+        else:
+            return "<foreign row>"
 
 
 def get_schema_from_path(schema_path):
@@ -918,13 +858,3 @@ def get_schema_from_path(schema_path):
     schema_module = importlib.util.module_from_spec(spec)
     spec.loader.exec_module(schema_module)
     return schema_module.SCHEMA
-
-
-def get_schema_map_from_path(schema_path):
-    return schema_to_map(get_schema_from_path(schema_path))
-
-
-def schema_to_map(schema):
-    if schema is None:
-        return {}
-    return collections.OrderedDict((table.name, table) for table in schema)
