@@ -1,6 +1,4 @@
 import collections
-import decimal
-import re
 import sqlite3
 import textwrap
 from abc import ABC
@@ -9,9 +7,9 @@ import sqliteparser
 from attr import attrib, attrs
 from sqliteparser import ast, quote
 
-CURRENT_TIMESTAMP = "STRFTIME('%Y-%m-%d %H:%M:%f000+00:00', 'now')"
-AUTO_TIMESTAMP = ("created_at", "last_updated_at")
-AUTO_TIMESTAMP_UPDATE_ONLY = ("last_updated_at",)
+CURRENT_TIMESTAMP_SQL = "STRFTIME('%Y-%m-%d %H:%M:%f', 'now')"
+AUTO_TIMESTAMP_DEFAULT = ("created_at", "last_updated_at")
+AUTO_TIMESTAMP_UPDATE_DEFAULT = ("last_updated_at",)
 
 
 # Sentinel object to detect keyword arguments that were not specified by the caller.
@@ -19,30 +17,44 @@ _Unset = object()
 
 
 class Database:
+    """
+    A class to represent a connection to a SQLite database. Typically used as a context
+    manager::
+
+        with Database("db.sqlite3") as db:
+            ...
+
+    On creation, the ``Database`` connection will open a SQL transaction which will be
+    either committed or rolled back by the context manager, depending on whether an
+    exception occurs.
+
+    You can also use multiple transactions over the life of the connection::
+
+        with Database("db.sqlite3", transaction=False) as db:
+            with db.transaction():
+                ...
+
+            with db.transaction():
+                ...
+    """
+
     def __init__(
         self,
         path,
         *,
-        schema=None,
         transaction=True,
         debugger=None,
         readonly=None,
         uri=False,
         cached_statements=_Unset,
+        create_auto_timestamp_columns=[],
+        update_auto_timestamp_columns=[],
     ):
         """
         Initialize a ``Database`` object.
 
-        Typical usage::
-
-            with Database("db.sqlite3") as db:
-                ...
-
         :param path: The path to the database file. You may pass ``":memory"`` for an
             in-memory database.
-        :param schema: A list of ``Table`` objects describing the schema of the
-            database. The schema is optional, but is required for certain features, such
-            as the ``get_related`` parameter of ``Database.get`` and ``Database.list``.
         :param transaction: If true, a transaction is automatically opened with BEGIN.
             When the ``Database`` class is used in a ``with`` statement, the transaction
             will be committed at the end (or rolled back if an exception occurs), so
@@ -72,6 +84,14 @@ class Database:
         :param uri: If true, the ``path`` argument is interpreted as a URI rather than a
             file path.
         :param cached_statements: Passed on to ``sqlite3.connect``.
+        :param create_auto_timestamp_columns: A default value for
+            ``auto_timestamp_columns`` in ``create`` and ``create_many``. Usually set to
+            ``["created_at", "last_updated_at"]`` in conjunction with a schema defined
+            using ``AutoTable``.
+        :param update_auto_timestamp_columns: A default value for
+            ``auto_timestamp_columns`` in ``update``. Usually set to
+            ``["last_updated_at"]`` in conjunction with a schema defined using
+            ``AutoTable``.
         """
         # Validate arguments.
         if readonly is not None:
@@ -91,6 +111,9 @@ class Database:
             else:
                 path = f"file:{path}"
 
+        self.create_auto_timestamp_columns = create_auto_timestamp_columns
+        self.update_auto_timestamp_columns = update_auto_timestamp_columns
+
         # We have to do this instead of passing the keyword arguments directly to
         # `sqlite3.connect` because we conditionally pass `cached_statements` if the
         # caller of `Database.__init__` gave it an explicit value.
@@ -114,12 +137,6 @@ class Database:
             debugger = PrintDebugger()
         self.debugger = debugger
 
-        self.schema = (
-            collections.OrderedDict((table.name, table) for table in schema)
-            if schema is not None
-            else None
-        )
-
         self.connection.row_factory = ordered_dict_row_factory
         self.cursor = self.connection.cursor()
 
@@ -129,6 +146,8 @@ class Database:
         if transaction:
             self.sql("BEGIN")
 
+        self.refresh_schema()
+
     def list(
         self,
         table,
@@ -197,12 +216,6 @@ class Database:
         where_clause = f"WHERE {where}" if where else ""
 
         if get_related:
-            if self.schema is None:
-                raise ISqliteApiError(
-                    "`get_related` requires that the database was initialized with a "
-                    + "schema."
-                )
-
             columns, joins = self._get_related_columns_and_joins(table, get_related)
             rows = self.sql(
                 f"SELECT {columns} FROM {quote(table)} {joins} {where_clause}"
@@ -233,12 +246,6 @@ class Database:
         where_clause = f"WHERE {where}" if where else ""
 
         if get_related:
-            if self.schema is None:
-                raise ISqliteApiError(
-                    "`get_related` requires that the database was initialized with a "
-                    + "schema."
-                )
-
             columns, joins = self._get_related_columns_and_joins(table, get_related)
             row = self.sql(
                 f"SELECT {columns} FROM {quote(table)} {joins} {where_clause}",
@@ -313,7 +320,7 @@ class Database:
         )
         return result[0]
 
-    def create(self, table, data, *, auto_timestamp=AUTO_TIMESTAMP):
+    def create(self, table, data, *, auto_timestamp_columns=_Unset):
         """
         Insert a new row.
 
@@ -322,22 +329,21 @@ class Database:
             attacks.
         :param data: The row to insert, as a dictionary from column names to column
             values.
-        :param auto_timestamp: A list of columns into which to insert the current time,
-            as an ISO 8601 timestamp. To disable auto timestamps, pass None.
+        :param auto_timestamp_columns: A list of columns into which to insert the
+            current date and time, as an ISO 8601 timestamp. Defaults to the value of
+            ``create_auto_timestamp_columns`` passed to ``__init__`` if unset.
         """
-        if auto_timestamp is None or auto_timestamp is False:
-            auto_timestamp = []
-        elif auto_timestamp is True:
-            auto_timestamp = AUTO_TIMESTAMP
+        if auto_timestamp_columns is _Unset:
+            auto_timestamp_columns = self.create_auto_timestamp_columns
 
         keys = list(data.keys())
         placeholders = ",".join("?" for _ in range(len(keys)))
         values = list(data.values())
 
         extra = []
-        for timestamp_column in auto_timestamp:
+        for timestamp_column in auto_timestamp_columns:
             keys.append(timestamp_column)
-            extra.append(CURRENT_TIMESTAMP)
+            extra.append(CURRENT_TIMESTAMP_SQL)
 
         if extra:
             extra = (", " if data else "") + ", ".join(extra)
@@ -353,7 +359,7 @@ class Database:
         self.cursor.execute(sql, values)
         return self.cursor.lastrowid
 
-    def create_many(self, table, data, *, auto_timestamp=AUTO_TIMESTAMP):
+    def create_many(self, table, data, *, auto_timestamp_columns=[]):
         """
         Insert multiple rows at once.
 
@@ -364,21 +370,19 @@ class Database:
 
         but more efficient.
         """
-        if auto_timestamp is None or auto_timestamp is False:
-            auto_timestamp = []
-        elif auto_timestamp is True:
-            auto_timestamp = AUTO_TIMESTAMP
-
         if not data:
             return
 
+        if auto_timestamp_columns is _Unset:
+            auto_timestamp_columns = self.create_auto_timestamp_columns
+
         keys = list(data[0].keys())
         placeholders = ",".join("?" for _ in range(len(keys)))
 
         extra = []
-        for timestamp_column in auto_timestamp:
+        for timestamp_column in auto_timestamp_columns:
             keys.append(timestamp_column)
-            extra.append(CURRENT_TIMESTAMP)
+            extra.append(CURRENT_TIMESTAMP_SQL)
 
         if extra:
             extra = (", " if data else "") + ", ".join(extra)
@@ -401,7 +405,7 @@ class Database:
         *,
         where=None,
         values={},
-        auto_timestamp=AUTO_TIMESTAMP_UPDATE_ONLY,
+        auto_timestamp_columns=[],
     ):
         """
         Update an existing row.
@@ -413,24 +417,25 @@ class Database:
             values.
         :param where: Restrict the set of rows to update. Same as for ``Database.list``.
         :param values: Same as for ``Database.list``.
-        :param auto_timestamp: Same as for ``Database.create``.
+        :param auto_timestamp_columns: Same as for ``Database.create``, except that if
+            the same column appears in both ``values`` and ``auto_timestamp_columns``,
+            the timestamp will be inserted instead of the value. Defaults to the value
+            of ``update_auto_timestamp_columns`` passed to ``__init__`` if unset.
         """
-        if auto_timestamp is None or auto_timestamp is False:
-            auto_timestamp = []
-        elif auto_timestamp is True:
-            auto_timestamp = AUTO_TIMESTAMP_UPDATE_ONLY
+        if auto_timestamp_columns is _Unset:
+            auto_timestamp_columns = self.update_auto_timestamp_columns
 
         updates = []
         for key, value in data.items():
-            if key in auto_timestamp:
+            if key in auto_timestamp_columns:
                 continue
 
             placeholder = f"v{len(values)}"
             values[placeholder] = value
             updates.append(f"{quote(key)} = :{placeholder}")
 
-        for timestamp_column in auto_timestamp:
-            updates.append(f"{quote(timestamp_column)} = {CURRENT_TIMESTAMP}")
+        for timestamp_column in auto_timestamp_columns:
+            updates.append(f"{quote(timestamp_column)} = {CURRENT_TIMESTAMP_SQL}")
 
         updates = ", ".join(updates)
         where_clause = f"WHERE {where}" if where else ""
@@ -538,6 +543,7 @@ class Database:
             )
 
         self.sql(f"CREATE TABLE {quote(table_name)}({','.join(map(str, columns))})")
+        self.refresh_schema()
 
     def drop_table(self, table_name):
         """
@@ -546,6 +552,7 @@ class Database:
         :param table_name: The name of the table to drop.
         """
         self.sql(f"DROP TABLE {quote(table_name)}")
+        self.refresh_schema()
 
     def rename_table(self, old_table_name, new_table_name):
         """
@@ -554,6 +561,7 @@ class Database:
         self.sql(
             f"ALTER TABLE {quote(old_table_name)} RENAME TO {quote(new_table_name)}"
         )
+        self.refresh_schema()
 
     def add_column(self, table_name, column_def):
         """
@@ -563,6 +571,223 @@ class Database:
         :param column_def: The definition of the column to add, as raw SQL.
         """
         self.sql(f"ALTER TABLE {quote(table_name)} ADD COLUMN {column_def}")
+        self.refresh_schema()
+
+    def drop_column(self, table_name, column_name):
+        """
+        Drop a column from the database.
+        """
+        # ALTER TABLE ... DROP COLUMN is only supported since SQLite version 3.35, so we
+        # implement it by hand here.
+        create_table_statement = self._get_create_table_statement(table_name)
+        columns = [
+            str(column)
+            for column in create_table_statement.columns
+            if column.name != column_name
+        ]
+        if len(columns) == len(create_table_statement.columns):
+            raise ColumnDoesNotExistError(table_name, column_name)
+
+        select = ", ".join(
+            quote(c.name)
+            for c in create_table_statement.columns
+            if c.name != column_name
+        )
+        self._migrate_table(table_name, columns, select=select)
+        self.refresh_schema()
+
+    def reorder_columns(self, table_name, column_names):
+        """
+        Reorder the columns of a database table.
+
+        :param table_name: The table to reorder.
+        :param column_names: The new order of the columns, as a list of strings. The
+            column names must be the same as in the database; otherwise, an exception
+            will be raised.
+        """
+        create_table_statement = self._get_create_table_statement(table_name)
+        column_map = collections.OrderedDict(
+            (c.name, c) for c in create_table_statement.columns
+        )
+
+        if set(column_names) != set(column_map.keys()):
+            raise ISqliteError(
+                "The set of reordered columns is not the same as the set of original "
+                + "columns."
+            )
+
+        columns = [str(column_map[name]) for name in column_names]
+        self._migrate_table(
+            table_name, columns, select=", ".join(map(quote, column_names))
+        )
+        self.refresh_schema()
+
+    def alter_column(self, table_name, column_name, new_column):
+        """
+        Alter the definition of a column.
+
+        :param table_name: The table to alter.
+        :param column_name: The column to alter.
+        :param new_column: The new definition of the column, without the name, as a SQL
+            string.
+        """
+        create_table_statement = self._get_create_table_statement(table_name)
+        columns = []
+        altered = False
+        for column in create_table_statement.columns:
+            if column.name == column_name:
+                columns.append(f"{column_name} {new_column}")
+                altered = True
+            else:
+                columns.append(str(column))
+
+        if not altered:
+            raise ColumnDoesNotExistError(table_name, column_name)
+
+        self._migrate_table(
+            table_name,
+            columns,
+            select=", ".join(quote(c.name) for c in create_table_statement.columns),
+        )
+        self.refresh_schema()
+
+    def rename_column(self, table_name, old_column_name, new_column_name):
+        """
+        Rename a column.
+        """
+        # ALTER TABLE ... RENAME COLUMN is only supported since SQLite version 3.25, so
+        # we implement it by hand here.
+        create_table_statement = self._get_create_table_statement(table_name)
+        columns_before = create_table_statement.columns
+        columns_after = []
+        altered = False
+        for column in create_table_statement.columns:
+            if column.name == old_column_name:
+                definition = (
+                    str(column.definition) if column.definition is not None else ""
+                )
+                columns_after.append(f"{new_column_name} {definition}")
+                altered = True
+            else:
+                columns_after.append(str(column))
+
+        if not altered:
+            raise ColumnDoesNotExistError(table_name, old_column_name)
+
+        self._migrate_table(
+            table_name,
+            columns_after,
+            select=", ".join(quote(c.name) for c in columns_before),
+        )
+        self.refresh_schema()
+
+    def diff(self, schema, *, table=None):
+        """
+        Return a list of differences between the Python schema and the actual database
+        schema.
+
+        :param schema: The Python schema to compare against the database.
+        :param table: The table to diff. If None, the entire database will be diffed.
+        """
+        schema = collections.OrderedDict((table.name, table) for table in schema)
+
+        tables_in_db = self._get_sql_schema()
+        tables_in_schema = schema.values() if table is None else [schema[table]]
+
+        diff = collections.defaultdict(list)
+        for table_in_schema in tables_in_schema:
+            name = table_in_schema.name
+            if name in tables_in_db:
+                columns_in_database = tables_in_db.pop(table_in_schema.name).columns
+                columns_in_schema = [
+                    column.as_sql() for column in table_in_schema.columns.values()
+                ]
+                self._diff_table(diff, name, columns_in_database, columns_in_schema)
+            else:
+                diff[table_in_schema.name].append(
+                    CreateTableMigration(
+                        table_in_schema.name,
+                        [
+                            str(column.as_sql())
+                            for column in table_in_schema.columns.values()
+                        ],
+                    )
+                )
+
+        if table is None:
+            for name in tables_in_db:
+                diff[name].append(DropTableMigration(name))
+
+        return diff
+
+    def apply_diff(self, diff):
+        """
+        Apply the diff returned by ``Database.diff`` to the database.
+
+        WARNING: This may cause columns or entire tables to be dropped from the
+        database. Make sure to examine the diff before applying it, e.g. by using the
+        ``isqlite migrate`` command.
+
+        The entire operation will occur in a transaction.
+
+        :param diff: A list of differences, as returned by ``Database.diff``.
+        """
+        with self.transaction(disable_foreign_keys=True):
+            for table_diff in diff.values():
+                for op in table_diff:
+                    if isinstance(op, CreateTableMigration):
+                        self.create_table(op.table_name, op.columns)
+                    elif isinstance(op, DropTableMigration):
+                        self.drop_table(op.table_name)
+                    elif isinstance(op, AlterColumnMigration):
+                        self.alter_column(
+                            op.table_name,
+                            op.column.name,
+                            str(op.column.definition)
+                            if op.column.definition is not None
+                            else "",
+                        )
+                    elif isinstance(op, AddColumnMigration):
+                        self.add_column(op.table_name, op.column)
+                    elif isinstance(op, DropColumnMigration):
+                        self.drop_column(op.table_name, op.column_name)
+                    elif isinstance(op, ReorderColumnsMigration):
+                        self.reorder_columns(op.table_name, op.column_names)
+                    elif isinstance(op, RenameColumnMigration):
+                        self.rename_column(
+                            op.table_name, op.old_column_name, op.new_column_name
+                        )
+                    else:
+                        raise ISqliteError("unknown migration op type")
+
+            self.refresh_schema()
+
+    def migrate(self, schema):
+        """
+        Migrate the database to match the Python schema.
+
+        WARNING: This may cause columns or entire tables to be dropped from the
+        database.
+
+        The entire operation will occur in a transaction.
+
+        :param schema: The Python schema to compare against the database.
+        """
+        self.apply_diff(self.diff(schema))
+
+    def refresh_schema(self):
+        """
+        Refresh the database's internal representation of the SQL schema.
+
+        Users do not normally need to call this function, as all the schema-altering
+        methods on this class already call it automatically. But if you alter the schema
+        using ``Database.sql`` or in an external database connection, you may need to
+        call this method for correct behavior.
+
+        The internal schema is used by the ``get_related`` functionality of ``list`` and
+        ``get``.
+        """
+        self.schema = self._get_sql_schema()
 
     def transaction(self, *, disable_foreign_keys=False):
         """
@@ -645,59 +870,6 @@ class Database:
 
         self.close()
 
-    def diff(self, schema=None, *, table=None):
-        """
-        Return a list of differences between the Python schema and the actual database
-        schema.
-
-        :param schema: The Python schema to use. If None, then the schema the database
-            was initialized with will be used.
-        :param table: The table to diff. If None, the entire database will be diffed.
-        """
-        if schema is None:
-            if self.schema is None:
-                raise ISqliteApiError(
-                    "`Database.diff` requires either that the database was initialized "
-                    + "with a schema, or an explicit schema was passed as a parameter."
-                )
-
-            schema = self.schema
-
-        tables_in_db = {
-            row["name"]: row["sql"]
-            for row in self.list(
-                "sqlite_master", where="type = 'table' AND NOT name LIKE 'sqlite_%'"
-            )
-        }
-        tables_in_schema = schema.values() if table is None else [schema[table]]
-
-        diff = collections.defaultdict(list)
-        for table_in_schema in tables_in_schema:
-            name = table_in_schema.name
-            if name in tables_in_db:
-                sql = tables_in_db.pop(table_in_schema.name)
-                columns_in_database = sqliteparser.parse(sql)[0].columns
-                columns_in_schema = [
-                    column.as_sql() for column in table_in_schema.columns.values()
-                ]
-                self._diff_table(diff, name, columns_in_database, columns_in_schema)
-            else:
-                diff[table_in_schema.name].append(
-                    CreateTableMigration(
-                        table_in_schema.name,
-                        [
-                            str(column.as_sql())
-                            for column in table_in_schema.columns.values()
-                        ],
-                    )
-                )
-
-        if table is None:
-            for name in tables_in_db:
-                diff[name].append(DropTableMigration(name))
-
-        return diff
-
     def _diff_table(self, diff, table_name, columns_in_database, columns_in_schema):
         columns_in_database_map = {
             column.name: i for i, column in enumerate(columns_in_database)
@@ -748,164 +920,6 @@ class Database:
 
         return diff
 
-    def apply_diff(self, diff):
-        """
-        Apply the diff returned by ``Database.diff`` to the database.
-
-        WARNING: This may cause columns or entire tables to be dropped from the
-        database. Make sure to examine the diff before applying it, e.g. by using the
-        ``isqlite migrate`` command.
-
-        The entire operation will occur in a transaction.
-
-        :param diff: A list of differences, as returned by ``Database.diff``.
-        """
-        with self.transaction(disable_foreign_keys=True):
-            for table_diff in diff.values():
-                for op in table_diff:
-                    if isinstance(op, CreateTableMigration):
-                        self.create_table(op.table_name, op.columns)
-                    elif isinstance(op, DropTableMigration):
-                        self.drop_table(op.table_name)
-                    elif isinstance(op, AlterColumnMigration):
-                        self.alter_column(
-                            op.table_name,
-                            op.column.name,
-                            str(op.column.definition)
-                            if op.column.definition is not None
-                            else "",
-                        )
-                    elif isinstance(op, AddColumnMigration):
-                        self.add_column(op.table_name, op.column)
-                    elif isinstance(op, DropColumnMigration):
-                        self.drop_column(op.table_name, op.column_name)
-                    elif isinstance(op, ReorderColumnsMigration):
-                        self.reorder_columns(op.table_name, op.column_names)
-                    elif isinstance(op, RenameColumnMigration):
-                        self.rename_column(
-                            op.table_name, op.old_column_name, op.new_column_name
-                        )
-                    else:
-                        raise ISqliteError("unknown migration op type")
-
-    def migrate(self, schema=None):
-        """
-        Migrate the database to match the Python schema.
-
-        WARNING: This may cause columns or entire tables to be dropped from the
-        database.
-
-        The entire operation will occur in a transaction.
-
-        :param schema: The Python schema to use. If None, then the schema the database
-            was initialized with will be used.
-        """
-        self.apply_diff(self.diff(schema))
-
-    def drop_column(self, table_name, column_name):
-        """
-        Drop a column from the database.
-        """
-        # ALTER TABLE ... DROP COLUMN is only supported since SQLite version 3.35, so we
-        # implement it by hand here.
-        create_table_statement = self._get_create_table_statement(table_name)
-        columns = [
-            str(column)
-            for column in create_table_statement.columns
-            if column.name != column_name
-        ]
-        if len(columns) == len(create_table_statement.columns):
-            raise ColumnDoesNotExistError(table_name, column_name)
-
-        select = ", ".join(
-            quote(c.name)
-            for c in create_table_statement.columns
-            if c.name != column_name
-        )
-        self._migrate_table(table_name, columns, select=select)
-
-    def reorder_columns(self, table_name, column_names):
-        """
-        Reorder the columns of a database table.
-
-        :param table_name: The table to reorder.
-        :param column_names: The new order of the columns, as a list of strings. The
-            column names must be the same as in the database; otherwise, an exception
-            will be raised.
-        """
-        create_table_statement = self._get_create_table_statement(table_name)
-        column_map = collections.OrderedDict(
-            (c.name, c) for c in create_table_statement.columns
-        )
-
-        if set(column_names) != set(column_map.keys()):
-            raise ISqliteError(
-                "The set of reordered columns is not the same as the set of original "
-                + "columns."
-            )
-
-        columns = [str(column_map[name]) for name in column_names]
-        self._migrate_table(
-            table_name, columns, select=", ".join(map(quote, column_names))
-        )
-
-    def alter_column(self, table_name, column_name, new_column):
-        """
-        Alter the definition of a column.
-
-        :param table_name: The table to alter.
-        :param column_name: The column to alter.
-        :param new_column: The new definition of the column, without the name, as a SQL
-            string.
-        """
-        create_table_statement = self._get_create_table_statement(table_name)
-        columns = []
-        altered = False
-        for column in create_table_statement.columns:
-            if column.name == column_name:
-                columns.append(f"{column_name} {new_column}")
-                altered = True
-            else:
-                columns.append(str(column))
-
-        if not altered:
-            raise ColumnDoesNotExistError(table_name, column_name)
-
-        self._migrate_table(
-            table_name,
-            columns,
-            select=", ".join(quote(c.name) for c in create_table_statement.columns),
-        )
-
-    def rename_column(self, table_name, old_column_name, new_column_name):
-        """
-        Rename a column.
-        """
-        # ALTER TABLE ... RENAME COLUMN is only supported since SQLite version 3.25, so
-        # we implement it by hand here.
-        create_table_statement = self._get_create_table_statement(table_name)
-        columns_before = create_table_statement.columns
-        columns_after = []
-        altered = False
-        for column in create_table_statement.columns:
-            if column.name == old_column_name:
-                definition = (
-                    str(column.definition) if column.definition is not None else ""
-                )
-                columns_after.append(f"{new_column_name} {definition}")
-                altered = True
-            else:
-                columns_after.append(str(column))
-
-        if not altered:
-            raise ColumnDoesNotExistError(table_name, old_column_name)
-
-        self._migrate_table(
-            table_name,
-            columns_after,
-            select=", ".join(quote(c.name) for c in columns_before),
-        )
-
     def _migrate_table(self, name, columns, *, select):
         # This procedure is copied from https://sqlite.org/lang_altertable.html
         # Create the new table under a temporary name.
@@ -939,38 +953,39 @@ class Database:
         if get_related is True:
             get_related = {
                 column.name
-                for column in table_schema.columns.values()
-                if isinstance(column, ForeignKeyColumn)
+                for column in table_schema.columns
+                if is_foreign_key_column(column)
                 # Don't fetch recursive relations because this will cause 'ambiguous
                 # column' errors in the SQL query.
-                and column.model != table
+                and get_foreign_key_model(column) != table
             }
         else:
             get_related = set(get_related)
 
         columns = []
         joins = []
-        for column in table_schema.columns.values():
+        for column in table_schema.columns:
             if column.name in get_related:
                 # Remove the column from the set so that we can check for any
                 # non-existent columns at the end.
                 get_related.remove(column.name)
 
-                if not isinstance(column, ForeignKeyColumn):
+                if not is_foreign_key_column(column):
                     raise ISqliteError(
                         f"{column.name!r} was passed in `get_related`, "
                         + "but it is not a foreign key column"
                     )
 
-                related_table_schema = self.schema[column.model]
-                for related_column in related_table_schema.columns.values():
+                foreign_table = get_foreign_key_model(column)
+                related_table_schema = self.schema[foreign_table]
+                for related_column in related_table_schema.columns:
                     name = f"{column.name}____{related_column.name}"
                     columns.append(
-                        f"{quote(column.model)}.{quote(related_column.name)} "
+                        f"{quote(foreign_table)}.{quote(related_column.name)} "
                         + f"AS {quote(name)}"
                     )
 
-                joins.append((column.name, column.model))
+                joins.append((column.name, foreign_table))
             else:
                 columns.append(f"{quote(table)}.{quote(column.name)}")
 
@@ -988,6 +1003,15 @@ class Database:
         )
         return columns, joins
 
+    def _get_sql_schema(self):
+        tables_in_db = {
+            row["name"]: sqliteparser.parse(row["sql"])[0]
+            for row in self.list(
+                "sqlite_master", where="type = 'table' AND NOT name LIKE 'sqlite_%'"
+            )
+        }
+        return tables_in_db
+
 
 class TransactionContextManager:
     def __init__(self, db, *, disable_foreign_keys=False):
@@ -1065,6 +1089,21 @@ def ordered_dict_row_factory(cursor, row):
     return r
 
 
+def is_foreign_key_column(column: sqliteparser.ast.Column) -> bool:
+    return any(
+        isinstance(constraint, sqliteparser.ast.ForeignKeyConstraint)
+        for constraint in column.definition.constraints
+    )
+
+
+def get_foreign_key_model(column: sqliteparser.ast.Column) -> str:
+    for constraint in column.definition.constraints:
+        if isinstance(constraint, sqliteparser.ast.ForeignKeyConstraint):
+            return constraint.foreign_table
+
+    return None
+
+
 class BaseColumn(ABC):
     def __init__(
         self, name, *, required=False, choices=[], default=None, sql_constraints=[]
@@ -1075,23 +1114,6 @@ class BaseColumn(ABC):
         self.default = default
         self.sql_constraints = sql_constraints[:]
 
-    def validate(self, v):
-        if v == "":
-            if self.default is not None:
-                return self.default, True
-            elif self.required:
-                return None, False
-            else:
-                return None, True
-
-        if self.choices:
-            return v, v in self.choices
-
-        return self._validate(v)
-
-    def _validate(self, v):
-        return v, True
-
     def as_sql(self):
         constraints = []
         if self.required:
@@ -1131,24 +1153,6 @@ class BaseColumn(ABC):
     def __str__(self):
         return str(self.as_sql())
 
-    def description(self):
-        sb = StringBuilder()
-        sb.text(self.name)
-        sb.text(" (")
-        sb.text(self.type.lower())
-        if not self.required:
-            sb.text(", optional")
-        if self.choices:
-            sb.text(f", choices = [{', '.join(map(repr, self.choices))}]")
-        sb.text(self._extra_description())
-        if self.default is not None and self.default != "":
-            sb.text(f", default = {self.default!r}")
-        sb.text(")")
-        return sb.build()
-
-    def _extra_description(self):
-        return ""
-
 
 class TextColumn(BaseColumn):
     type = "TEXT"
@@ -1158,16 +1162,6 @@ class TextColumn(BaseColumn):
         if not self.required and self.default is None:
             self.default = ""
 
-    def validate(self, *args, **kwargs):
-        # Normally, subclasses of BaseColumn will override _validate instead. But
-        # TextColumn needs to make sure that validate _always_ returns "" instead of
-        # None, so it needs to override the default validate method instead.
-        v, is_valid = super().validate(*args, **kwargs)
-        if v is None and is_valid:
-            return "", True
-        else:
-            return v, is_valid
-
     def as_sql(self):
         constraints = [not_null_constraint()]
         if self.required:
@@ -1214,111 +1208,42 @@ class IntegerColumn(BaseColumn):
                 check_operator_constraint(self.name, ">=", ast.Integer(self.min))
             )
 
-    def _validate(self, v):
-        try:
-            v = int(v)
-        except ValueError:
-            return None, False
-
-        if self.min is not None and v < self.min:
-            return None, False
-
-        if self.max is not None and v > self.max:
-            return None, False
-
-        return v, True
-
-    def _extra_description(self):
-        sb = StringBuilder()
-        if self.min is not None:
-            sb.text(f", min = {self.min}")
-        if self.max is not None:
-            sb.text(f", max = {self.max}")
-        return sb.build()
-
 
 class BooleanColumn(BaseColumn):
     type = "BOOLEAN"
 
-    def _validate(self, v):
-        if v == "1" or v.lower() == "true":
-            return True, True
-        elif v == "0" or v.lower() == "false":
-            return False, True
-        else:
-            return None, False
-
 
 class DateColumn(BaseColumn):
     type = "DATE"
-    date_pattern = re.compile(r"^[0-9]{4}-[0-9]{2}-[0-9]{2}$")
-
-    def _validate(self, v):
-        if self.date_pattern.match(v):
-            return v, True
-        else:
-            return None, False
 
 
 class TimestampColumn(BaseColumn):
     type = "TIMESTAMP"
-    pattern = re.compile(
-        r"^[0-9]{4}-[0-9]{2}-[0-9]{2} "
-        + r"[0-9]{1,2}:[0-9]{2}:[0-9]{2}\.[0-9]{6}\+[0-9]{2}:[0-9]{2}$"
-    )
-
-    def _validate(self, v):
-        if self.pattern.match(v):
-            return v, True
-        else:
-            return None, False
 
 
 class TimeColumn(BaseColumn):
     type = "TIME"
-    pattern = re.compile(r"^[0-9]{1,2}:[0-9]{2}$")
-
-    def _validate(self, v):
-        if self.pattern.match(v):
-            return v, True
-        else:
-            return None, False
 
 
 class DecimalColumn(BaseColumn):
     type = "DECIMAL"
 
-    def _validate(self, v):
-        try:
-            return decimal.Decimal(v), True
-        except decimal.InvalidOperation:
-            return None, False
-
 
 class ForeignKeyColumn(BaseColumn):
     type = "INTEGER"
 
-    def __init__(self, *args, model, on_delete=ast.OnDelete.SET_NULL, **kwargs):
+    def __init__(self, *args, foreign_table, on_delete=ast.OnDelete.SET_NULL, **kwargs):
         super().__init__(*args, **kwargs)
-        self.model = model
+        self.foreign_table = foreign_table
         self.sql_constraints.append(
             ast.ForeignKeyConstraint(
                 columns=[],
-                foreign_table=self.model,
+                foreign_table=self.foreign_table,
                 foreign_columns=[],
                 on_delete=on_delete,
             )
         )
 
-    def _validate(self, v):
-        try:
-            return int(v), True
-        except ValueError:
-            return None, False
-
-    def _extra_description(self):
-        return f", foreign key = {self.model}"
-
 
 class PrimaryKeyColumn(IntegerColumn):
     def __init__(self, name):
@@ -1344,27 +1269,20 @@ def check_operator_constraint(name, operator, value):
 
 
 class Table:
-    def __init__(self, name, columns, *, as_string=None):
+    def __init__(self, name, columns):
         self.name = name
         self.columns = collections.OrderedDict(
             (column.name, column) for column in columns
         )
-        self.as_string = (
-            as_string if as_string is not None else lambda row: str(row["id"])
-        )
 
 
 class AutoTable(Table):
-    def __init__(self, name, columns, **kwargs):
-        id_column = IntegerColumn(
-            "id",
-            required=True,
-            sql_constraints=[ast.PrimaryKeyConstraint(autoincrement=True)],
-        )
+    def __init__(self, name, columns):
+        id_column = PrimaryKeyColumn("id")
         created_at_column = TimestampColumn("created_at", required=True)
         last_updated_at_column = TimestampColumn("last_updated_at", required=True)
         columns = [id_column] + columns + [created_at_column, last_updated_at_column]
-        super().__init__(name, columns, **kwargs)
+        super().__init__(name, columns)
 
 
 def convert_default(default):
@@ -1461,17 +1379,6 @@ class PrintDebugger:
         print("=== END SQL DEBUGGER ===")
 
 
-class StringBuilder:
-    def __init__(self):
-        self.parts = []
-
-    def text(self, s):
-        self.parts.append(s)
-
-    def build(self):
-        return "".join(self.parts)
-
-
 class ISqliteError(Exception):
     pass
 
